@@ -1,3 +1,4 @@
+require 'benchmark'
 require 'yaml'
 require 'capistrano/recipes/deploy/scm'
 require 'capistrano/recipes/deploy/strategy'
@@ -27,6 +28,11 @@ _cset :deploy_via, :checkout
 _cset(:deploy_to) { "/u/apps/#{application}" }
 _cset(:revision)  { source.head }
 
+_cset :rails_env, "production"
+_cset :rake, "rake"
+
+_cset :maintenance_basename, "maintenance"
+
 # =========================================================================
 # These variables should NOT be changed unless you are very confident in
 # what you are doing. Make sure you understand all the implications of your
@@ -38,6 +44,7 @@ _cset(:real_revision)     { source.local.query_revision(revision) { |cmd| with_e
 
 _cset(:strategy)          { Capistrano::Deploy::Strategy.new(deploy_via, self) }
 
+# If overriding release name, please also select an appropriate setting for :releases below.
 _cset(:release_name)      { set :deploy_timestamped, true; Time.now.utc.strftime("%Y%m%d%H%M%S") }
 
 _cset :version_dir,       "releases"
@@ -50,13 +57,13 @@ _cset(:shared_path)       { File.join(deploy_to, shared_dir) }
 _cset(:current_path)      { File.join(deploy_to, current_dir) }
 _cset(:release_path)      { File.join(releases_path, release_name) }
 
-_cset(:releases)          { capture("ls -xt #{releases_path}").split.reverse }
-_cset(:current_release)   { File.join(releases_path, releases.last) }
+_cset(:releases)          { capture("ls -x #{releases_path}", :except => { :no_release => true }).split.sort }
+_cset(:current_release)   { releases.length > 0 ? File.join(releases_path, releases.last) : nil }
 _cset(:previous_release)  { releases.length > 1 ? File.join(releases_path, releases[-2]) : nil }
 
-_cset(:current_revision)  { capture("cat #{current_path}/REVISION").chomp }
-_cset(:latest_revision)   { capture("cat #{current_release}/REVISION").chomp }
-_cset(:previous_revision) { capture("cat #{previous_release}/REVISION").chomp }
+_cset(:current_revision)  { capture("cat #{current_path}/REVISION",     :except => { :no_release => true }).chomp }
+_cset(:latest_revision)   { capture("cat #{current_release}/REVISION",  :except => { :no_release => true }).chomp }
+_cset(:previous_revision) { capture("cat #{previous_release}/REVISION", :except => { :no_release => true }).chomp if previous_release }
 
 _cset(:run_method)        { fetch(:use_sudo, true) ? :sudo : :run }
 
@@ -94,9 +101,18 @@ end
 # returns the command output as a string
 def run_locally(cmd)
   logger.trace "executing locally: #{cmd.inspect}" if logger
-  `#{cmd}`
+  output_on_stdout = nil
+  elapsed = Benchmark.realtime do
+    output_on_stdout = `#{cmd}`
+  end
+  if $?.to_i > 0 # $? is command exit code (posix style)
+    raise Capistrano::LocalArgumentError, "Command #{cmd} returned status code #{$?}"
+  end
+  logger.trace "command finished in #{(elapsed * 1000).round}ms" if logger
+  output_on_stdout
 end
 
+
 # If a command is given, this will try to execute the given command, as
 # described below. Otherwise, it will return a string for use in embedding in
 # another command, for executing that command as described below.
@@ -169,7 +185,8 @@ namespace :deploy do
   task :setup, :except => { :no_release => true } do
     dirs = [deploy_to, releases_path, shared_path]
     dirs += shared_children.map { |d| File.join(shared_path, d) }
-    run "#{try_sudo} mkdir -p #{dirs.join(' ')} && #{try_sudo} chmod g+w #{dirs.join(' ')}"
+    run "#{try_sudo} mkdir -p #{dirs.join(' ')}"
+    run "#{try_sudo} chmod g+w #{dirs.join(' ')}" if fetch(:group_writable, true)
   end
 
   desc <<-DESC
@@ -218,7 +235,8 @@ namespace :deploy do
     public/stylesheets, and public/javascripts so that the times are \
     consistent (so that asset timestamping works).  This touch process \
     is only carried out if the :normalize_asset_timestamps variable is \
-    set to true, which is the default.
+    set to true, which is the default The asset directories can be overridden \
+    using the :public_children variable.
   DESC
   task :finalize_update, :except => { :no_release => true } do
     run "chmod -R g+w #{latest_release}" if fetch(:group_writable, true)
@@ -236,7 +254,7 @@ namespace :deploy do
 
     if fetch(:normalize_asset_timestamps, true)
       stamp = Time.now.utc.strftime("%Y%m%d%H%M.%S")
-      asset_paths = %w(images stylesheets javascripts).map { |p| "#{latest_release}/public/#{p}" }.join(" ")
+      asset_paths = fetch(:public_children, %w(images stylesheets javascripts)).map { |p| "#{latest_release}/public/#{p}" }.join(" ")
       run "find #{asset_paths} -exec touch -t #{stamp} {} ';'; true", :env => { "TZ" => "UTC" }
     end
   end
@@ -288,18 +306,11 @@ namespace :deploy do
   end
 
   desc <<-DESC
-    Restarts your application. This works by calling the script/process/reaper \
-    script under the current path.
-    
-    By default, this will be invoked via sudo as the `app' user. If \
-    you wish to run it as a different user, set the :runner variable to \
-    that user. If you are in an environment where you can't use sudo, set \
-    the :use_sudo variable to false:
-    
-      set :use_sudo, false
+    Blank task exists as a hook into which to install your own environment \
+    specific behaviour.
   DESC
   task :restart, :roles => :app, :except => { :no_release => true } do
-    try_runner "#{current_path}/script/process/reaper"
+    # Empty Task to overload with your platform specifics
   end
 
   namespace :rollback do
@@ -364,18 +375,16 @@ namespace :deploy do
       set :migrate_target, :latest
   DESC
   task :migrate, :roles => :db, :only => { :primary => true } do
-    rake = fetch(:rake, "rake")
-    rails_env = fetch(:rails_env, "production")
     migrate_env = fetch(:migrate_env, "")
     migrate_target = fetch(:migrate_target, :latest)
 
     directory = case migrate_target.to_sym
       when :current then current_path
-      when :latest  then current_release
+      when :latest  then latest_release
       else raise ArgumentError, "unknown migration target #{migrate_target.inspect}"
       end
 
-    run "cd #{directory}; #{rake} RAILS_ENV=#{rails_env} #{migrate_env} db:migrate"
+    run "cd #{directory} && #{rake} RAILS_ENV=#{rails_env} #{migrate_env} db:migrate"
   end
 
   desc <<-DESC
@@ -467,35 +476,19 @@ namespace :deploy do
   end
 
   desc <<-DESC
-    Start the application servers. This will attempt to invoke a script \
-    in your application called `script/spin', which must know how to start \
-    your application listeners. For Rails applications, you might just have \
-    that script invoke `script/process/spawner' with the appropriate \
-    arguments.
-
-    By default, the script will be executed via sudo as the `app' user. If \
-    you wish to run it as a different user, set the :runner variable to \
-    that user. If you are in an environment where you can't use sudo, set \
-    the :use_sudo variable to false.
+    Blank task exists as a hook into which to install your own environment \
+    specific behaviour.
   DESC
   task :start, :roles => :app do
-    run "cd #{current_path} && #{try_runner} nohup script/spin"
+    # Empty Task to overload with your platform specifics
   end
 
   desc <<-DESC
-    Stop the application servers. This will call script/process/reaper for \
-    both the spawner process, and all of the application processes it has \
-    spawned. As such, it is fairly Rails specific and may need to be \
-    overridden for other systems.
-
-    By default, the script will be executed via sudo as the `app' user. If \
-    you wish to run it as a different user, set the :runner variable to \
-    that user. If you are in an environment where you can't use sudo, set \
-    the :use_sudo variable to false.
+    Blank task exists as a hook into which to install your own environment \
+    specific behaviour.
   DESC
   task :stop, :roles => :app do
-    run "if [ -f #{current_path}/tmp/pids/dispatch.spawner.pid ]; then #{try_runner} #{current_path}/script/process/reaper -a kill -r dispatch.spawner.pid; fi"
-    try_runner "#{current_path}/script/process/reaper -a kill"
+    # Empty Task to overload with your platform specifics
   end
 
   namespace :pending do
@@ -522,7 +515,7 @@ namespace :deploy do
   namespace :web do
     desc <<-DESC
       Present a maintenance page to visitors. Disables your application's web \
-      interface by writing a "maintenance.html" file to each web server. The \
+      interface by writing a "#{maintenance_basename}.html" file to each web server. The \
       servers must be configured to detect the presence of this file, and if \
       it is present, always display it instead of performing the request.
 
@@ -538,7 +531,20 @@ namespace :deploy do
     DESC
     task :disable, :roles => :web, :except => { :no_release => true } do
       require 'erb'
-      on_rollback { run "rm #{shared_path}/system/maintenance.html" }
+      on_rollback { run "rm #{shared_path}/system/#{maintenance_basename}.html" }
+
+      warn <<-EOHTACCESS
+
+        # Please add something like this to your site's htaccess to redirect users to the maintenance page.
+        # More Info: http://www.shiftcommathree.com/articles/make-your-rails-maintenance-page-respond-with-a-503
+
+        ErrorDocument 503 /system/#{maintenance_basename}.html
+        RewriteEngine On
+        RewriteCond %{REQUEST_URI} !\.(css|gif|jpg|png)$
+        RewriteCond %{DOCUMENT_ROOT}/system/#{maintenance_basename}.html -f
+        RewriteCond %{SCRIPT_FILENAME} !#{maintenance_basename}.html
+        RewriteRule ^.*$  -  [redirect=503,last]
+      EOHTACCESS
 
       reason = ENV['REASON']
       deadline = ENV['UNTIL']
@@ -546,17 +552,17 @@ namespace :deploy do
       template = File.read(File.join(File.dirname(__FILE__), "templates", "maintenance.rhtml"))
       result = ERB.new(template).result(binding)
 
-      put result, "#{shared_path}/system/maintenance.html", :mode => 0644
+      put result, "#{shared_path}/system/#{maintenance_basename}.html", :mode => 0644
     end
 
     desc <<-DESC
       Makes the application web-accessible again. Removes the \
-      "maintenance.html" page generated by deploy:web:disable, which (if your \
+      "#{maintenance_basename}.html" page generated by deploy:web:disable, which (if your \
       web servers are configured correctly) will make your application \
       web-accessible again.
     DESC
     task :enable, :roles => :web, :except => { :no_release => true } do
-      run "rm #{shared_path}/system/maintenance.html"
+      run "rm #{shared_path}/system/#{maintenance_basename}.html"
     end
   end
 end
